@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2023 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2024 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -294,7 +294,8 @@ uint32_t FAudio_CreateSourceVoice(
 
 	if (	pSourceFormat->wFormatTag == FAUDIO_FORMAT_PCM ||
 		pSourceFormat->wFormatTag == FAUDIO_FORMAT_IEEE_FLOAT ||
-		pSourceFormat->wFormatTag == FAUDIO_FORMAT_WMAUDIO2	)
+		pSourceFormat->wFormatTag == FAUDIO_FORMAT_WMAUDIO2 ||
+		pSourceFormat->wFormatTag == FAUDIO_FORMAT_WMAUDIO3	)
 	{
 		FAudioWaveFormatExtensible *fmtex = (FAudioWaveFormatExtensible*) audio->pMalloc(
 			sizeof(FAudioWaveFormatExtensible)
@@ -321,6 +322,10 @@ uint32_t FAudio_CreateSourceVoice(
 		{
 			FAudio_memcpy(&fmtex->SubFormat, &DATAFORMAT_SUBTYPE_WMAUDIO2, sizeof(FAudioGUID));
 		}
+		else if (pSourceFormat->wFormatTag == FAUDIO_FORMAT_WMAUDIO3)
+		{
+			FAudio_memcpy(&fmtex->SubFormat, &DATAFORMAT_SUBTYPE_WMAUDIO3, sizeof(FAudioGUID));
+		}
 		(*ppSourceVoice)->src.format = &fmtex->Format;
 	}
 	else if (pSourceFormat->wFormatTag == FAUDIO_FORMAT_MSADPCM)
