@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2023 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2024 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -323,13 +323,13 @@ void FAudio_OPERATIONSET_QueueSetEffectParameters(
 );
 void FAudio_OPERATIONSET_QueueSetFilterParameters(
 	FAudioVoice *voice,
-	const FAudioFilterParameters *pParameters,
+	const FAudioFilterParametersEXT *pParameters,
 	uint32_t OperationSet
 );
 void FAudio_OPERATIONSET_QueueSetOutputFilterParameters(
 	FAudioVoice *voice,
 	FAudioVoice *pDestinationVoice,
-	const FAudioFilterParameters *pParameters,
+	const FAudioFilterParametersEXT *pParameters,
 	uint32_t OperationSet
 );
 void FAudio_OPERATIONSET_QueueSetVolume(
@@ -433,7 +433,7 @@ struct FAudioVoice
 	float **sendCoefficients;
 	float **mixCoefficients;
 	FAudioMixCallback *sendMix;
-	FAudioFilterParameters *sendFilter;
+	FAudioFilterParametersEXT *sendFilter;
 	FAudioFilterState **sendFilterState;
 	struct
 	{
@@ -445,7 +445,7 @@ struct FAudioVoice
 		uint8_t *parameterUpdates;
 		uint8_t *inPlaceProcessing;
 	} effects;
-	FAudioFilterParameters filter;
+	FAudioFilterParametersEXT filter;
 	FAudioFilterState *filterState;
 	FAudioMutex sendLock;
 	FAudioMutex effectLock;
