@@ -1,6 +1,6 @@
 /* FAudio - XAudio Reimplementation for FNA
  *
- * Copyright (c) 2011-2023 Ethan Lee, Luigi Auriemma, and the MonoGame Team
+ * Copyright (c) 2011-2024 Ethan Lee, Luigi Auriemma, and the MonoGame Team
  *
  * This software is provided 'as-is', without any express or implied warranty.
  * In no event will the authors be held liable for any damages arising from
@@ -80,6 +80,8 @@
 #define STB_VORBIS_NO_INTEGER_CONVERSION 1
 #include "stb_vorbis.h"
 
+#include "qoa_decoder.h"
+
 /* Globals */
 
 static float songVolume = 1.0f;
@@ -88,8 +90,15 @@ static FAudioMasteringVoice *songMaster = NULL;
 
 static FAudioSourceVoice *songVoice = NULL;
 static FAudioVoiceCallback callbacks;
-static stb_vorbis *activeSong = NULL;
-static stb_vorbis_info activeSongInfo;
+static stb_vorbis *activeVorbisSong = NULL;
+static stb_vorbis_info activeVorbisSongInfo;
+
+static qoa *activeQoaSong = NULL;
+static unsigned int qoaChannels = 0;
+static unsigned int qoaSampleRate = 0;
+static unsigned int qoaSamplesPerChannelPerFrame = 0;
+static unsigned int qoaTotalSamplesPerChannel = 0;
+
 static uint8_t *songCache;
 
 /* Internal Functions */
@@ -97,20 +106,39 @@ static uint8_t *songCache;
 static void XNA_SongSubmitBuffer(FAudioVoiceCallback *callback, void *pBufferContext)
 {
 	FAudioBuffer buffer;
-	uint32_t decoded = stb_vorbis_get_samples_float_interleaved(
-		activeSong,
-		activeSongInfo.channels,
-		(float*) songCache,
-		activeSongInfo.sample_rate * activeSongInfo.channels
-	);
+	uint32_t decoded = 0;
+
+	if (activeVorbisSong != NULL)
+	{
+		decoded = stb_vorbis_get_samples_float_interleaved(
+			activeVorbisSong,
+			activeVorbisSongInfo.channels,
+			(float*) songCache,
+			activeVorbisSongInfo.sample_rate * activeVorbisSongInfo.channels
+		);
+		buffer.AudioBytes = decoded * activeVorbisSongInfo.channels * sizeof(float);
+		buffer.Flags = (decoded < activeVorbisSongInfo.sample_rate) ?
+			FAUDIO_END_OF_STREAM :
+			0;
+	}
+	else if (activeQoaSong != NULL)
+	{
+		/* TODO: decode multiple frames? */
+		decoded = qoa_decode_next_frame(
+			activeQoaSong,
+			(short*) songCache
+		);
+		buffer.AudioBytes = decoded * qoaChannels * sizeof(short);
+		buffer.Flags = (decoded < qoaSamplesPerChannelPerFrame) ?
+			FAUDIO_END_OF_STREAM :
+			0;
+	}
+
 	if (decoded == 0)
 	{
 		return;
 	}
-	buffer.Flags = (decoded < activeSongInfo.sample_rate) ?
-		FAUDIO_END_OF_STREAM :
-		0;
-	buffer.AudioBytes = decoded * activeSongInfo.channels * sizeof(float);
+
 	buffer.pAudioData = songCache;
 	buffer.PlayBegin = 0;
 	buffer.PlayLength = decoded;
@@ -138,10 +166,15 @@ static void XNA_SongKill()
 		FAudio_free(songCache);
 		songCache = NULL;
 	}
-	if (activeSong != NULL)
+	if (activeVorbisSong != NULL)
 	{
-		stb_vorbis_close(activeSong);
-		activeSong = NULL;
+		stb_vorbis_close(activeVorbisSong);
+		activeVorbisSong = NULL;
+	}
+	if (activeQoaSong != NULL)
+	{
+		qoa_close(activeQoaSong);
+		activeQoaSong = NULL;
 	}
 }
 
@@ -173,17 +206,39 @@ FAUDIOAPI float XNA_PlaySong(const char *name)
 	FAudioWaveFormatEx format;
 	XNA_SongKill();
 
-	activeSong = stb_vorbis_open_filename(name, NULL, NULL);
+	activeVorbisSong = stb_vorbis_open_filename(name, NULL, NULL);
 
-	/* Set format info */
-	activeSongInfo = stb_vorbis_get_info(activeSong);
-	format.wFormatTag = FAUDIO_FORMAT_IEEE_FLOAT;
-	format.nChannels = activeSongInfo.channels;
-	format.nSamplesPerSec = activeSongInfo.sample_rate;
-	format.wBitsPerSample = sizeof(float) * 8;
-	format.nBlockAlign = format.nChannels * format.wBitsPerSample / 8;
-	format.nAvgBytesPerSec = format.nSamplesPerSec * format.nBlockAlign;
-	format.cbSize = 0;
+	if (activeVorbisSong != NULL)
+	{
+		activeVorbisSongInfo = stb_vorbis_get_info(activeVorbisSong);
+		format.wFormatTag = FAUDIO_FORMAT_IEEE_FLOAT;
+		format.nChannels = activeVorbisSongInfo.channels;
+		format.nSamplesPerSec = activeVorbisSongInfo.sample_rate;
+		format.wBitsPerSample = sizeof(float) * 8;
+		format.nBlockAlign = format.nChannels * format.wBitsPerSample / 8;
+		format.nAvgBytesPerSec = format.nSamplesPerSec * format.nBlockAlign;
+		format.cbSize = 0;
+	}
+	else /* It's not vorbis, try qoa!*/
+	{
+		activeQoaSong = qoa_open_from_filename(name);
+
+		if (activeQoaSong == NULL)
+		{
+			/* It's neither vorbis nor qoa, time to bail */
+			return 0;
+		}
+
+		qoa_attributes(activeQoaSong, &qoaChannels, &qoaSampleRate, &qoaSamplesPerChannelPerFrame, &qoaTotalSamplesPerChannel);
+
+		format.wFormatTag = FAUDIO_FORMAT_PCM;
+		format.nChannels = qoaChannels;
+		format.nSamplesPerSec = qoaSampleRate;
+		format.wBitsPerSample = 16;
+		format.nBlockAlign = format.nChannels * format.wBitsPerSample / 8;
+		format.nAvgBytesPerSec = format.nSamplesPerSec * format.nBlockAlign;
+		format.cbSize = 0;
+	}
 
 	/* Allocate decode cache */
 	songCache = (uint8_t*) FAudio_malloc(format.nAvgBytesPerSec);
@@ -204,12 +259,30 @@ FAUDIOAPI float XNA_PlaySong(const char *name)
 	FAudioVoice_SetVolume(songVoice, songVolume, 0);
 
 	/* Okay, this song is decoding now */
-	stb_vorbis_seek_start(activeSong);
+	if (activeVorbisSong != NULL)
+	{
+		stb_vorbis_seek_start(activeVorbisSong);
+	}
+	else if (activeQoaSong != NULL)
+	{
+		qoa_seek_frame(activeQoaSong, 0);
+	}
+
 	XNA_SongSubmitBuffer(NULL, NULL);
 
 	/* Finally. */
 	FAudioSourceVoice_Start(songVoice, 0, 0);
-	return stb_vorbis_stream_length_in_seconds(activeSong);
+
+	if (activeVorbisSong != NULL)
+	{
+		return stb_vorbis_stream_length_in_seconds(activeVorbisSong);
+	}
+	else if (activeQoaSong != NULL)
+	{
+		return qoaTotalSamplesPerChannel / (float) qoaSampleRate;
+	}
+
+	return 0;
 }
 
 FAUDIOAPI void XNA_PauseSong()
@@ -247,7 +320,7 @@ FAUDIOAPI void XNA_SetSongVolume(float volume)
 FAUDIOAPI uint32_t XNA_GetSongEnded()
 {
 	FAudioVoiceState state;
-	if (songVoice == NULL || activeSong == NULL)
+	if (songVoice == NULL || (activeVorbisSong == NULL && activeQoaSong == NULL))
 	{
 		return 1;
 	}
